package net.wanji.opt.service.signalopt.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.databus.dao.mapper.CrossDataHistMapper;
import net.wanji.databus.dao.mapper.GreenwaveInfoMapper;
import net.wanji.databus.po.CrossDataHistPO;
import net.wanji.opt.dao.mapper.StrategyControlInfoMapper;
import net.wanji.opt.dao.mapper.StrategyGreenOptHistMapper;
import net.wanji.opt.po.StrategyGreenOptHistEntity;
import net.wanji.opt.service.signalopt.GreenBeltInfoService;
import net.wanji.opt.synthesis.pojo.StrategyControlDataEntity;
import net.wanji.opt.vo.GreenBeltFlowStopTimeVO;
import net.wanji.opt.vo.GreenBeltSpeedWidthVO;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.text.ParseException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneOffset;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/12/02 13:42
 */
@Service
@Slf4j
public class GreenBeltServiceImpl implements GreenBeltInfoService {

    @Resource
    private GreenwaveInfoMapper greenwaveInfoMapper;
    @Resource
    private CrossDataHistMapper crossDataHistMapper;
    @Resource
    private StrategyGreenOptHistMapper strategyGreenOptHistMapper;

    @Override
    public List<GreenBeltSpeedWidthVO> greenBeltSpeedWidth(Integer greenId) throws Exception {
        try {
            LambdaQueryWrapper<StrategyGreenOptHistEntity> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(StrategyGreenOptHistEntity::getGreenId, greenId);
            LocalDate currentDate = LocalDate.now();
            LocalTime startTime = LocalTime.MIDNIGHT;
            LocalDateTime startOfDay = LocalDateTime.of(currentDate, startTime);
            queryWrapper.ge(StrategyGreenOptHistEntity::getControlTime, startOfDay);
            List<StrategyGreenOptHistEntity> entities = strategyGreenOptHistMapper.selectList(queryWrapper);
            List<GreenBeltSpeedWidthVO> results = new ArrayList<>();
            if (!CollectionUtils.isEmpty(entities)) {
                for (StrategyGreenOptHistEntity entity : entities) {
                    GreenBeltSpeedWidthVO greenBeltSpeedWidthVO = new GreenBeltSpeedWidthVO();
                    Date date = DateUtil.parse(entity.getControlTime(), "yyyy-MM-dd HH:mm:ss");
                    greenBeltSpeedWidthVO.setStartTime(date);
                    greenBeltSpeedWidthVO.setSpeed(entity.getMaxSpeed());
                    greenBeltSpeedWidthVO.setWidth(entity.getGreenWidthTime());
                    results.add(greenBeltSpeedWidthVO);
                }
            }
            return results;
        } catch (Exception e) {
            log.error("绿波带宽曲线异常：", e);
            throw new RuntimeException(e);
        }
    }

    @Override
    public List<GreenBeltFlowStopTimeVO> greenBeltCrossDetailHist(Integer greenId) throws Exception {
        try {
            List<String> crossIds = greenwaveInfoMapper.selectCrossIdsById(greenId);
            long currentTimeMillis = System.currentTimeMillis();
            int startSecond = (int) (currentTimeMillis / 1000 - 3600);
            int endSecond = (int) (currentTimeMillis / 1000);
            List<CrossDataHistPO> crossDataHistPOS = crossDataHistMapper.selectByCrossIdsAndTimestamp(crossIds, startSecond , endSecond);
            if (!CollectionUtils.isEmpty(crossDataHistPOS)) {
                List<GreenBeltFlowStopTimeVO> results = new ArrayList<>();
                Map<Date, List<CrossDataHistPO>> startTimeMap = crossDataHistPOS.stream().collect(Collectors.groupingBy(CrossDataHistPO::getStartTime));
                for (Map.Entry<Date, List<CrossDataHistPO>> entry : startTimeMap.entrySet()) {
                    GreenBeltFlowStopTimeVO greenBeltFlowStopTimeVO = new GreenBeltFlowStopTimeVO();
                    Date startDate = entry.getKey();
                    List<GreenBeltFlowStopTimeVO.FlowStopTimeDetail> crossList = new ArrayList<>();
                    greenBeltFlowStopTimeVO.setStartTime(startDate);
                    List<CrossDataHistPO> value = entry.getValue();
                    if (!CollectionUtils.isEmpty(value)) {
                        for (CrossDataHistPO crossDataHistPO : value) {
                            GreenBeltFlowStopTimeVO.FlowStopTimeDetail flowStopTimeDetail = new GreenBeltFlowStopTimeVO.FlowStopTimeDetail();
                            flowStopTimeDetail.setFlow(crossDataHistPO.getFlow());
                            flowStopTimeDetail.setStopTimes(crossDataHistPO.getStopTimes());
                            flowStopTimeDetail.setCrossId(crossDataHistPO.getCrossId());
                            // todo 先删除，后续通过路口距离/速度计算 距离通过卫博发送数据
                            flowStopTimeDetail.setTravelTime(0.0);
                            crossList.add(flowStopTimeDetail);
                        }
                    }
                    greenBeltFlowStopTimeVO.setDetailList(crossList);
                    results.add(greenBeltFlowStopTimeVO);
                }
                Collections.sort(results, Comparator.comparing(GreenBeltFlowStopTimeVO::getStartTime));
                return results;
            }
            return Collections.EMPTY_LIST;
        } catch (Exception e) {
            log.error("绿波协调方向路口流量停车次数查询异常：", e);
            throw new RuntimeException(e);
        }
    }
}
